from tkinter import *
import pm_logic as pm


class GUIGameObject(pm.GameObject):
    """
    Adds a method draw_inplace() to the GameObject, that subclasses
    should redefine in an appropriate manner..
    """

    def __init__(self, cnv, **kwargs):
        """
        Added cnv parameter for GameApp Canvas.
        """
        super().__init__(**kwargs)
        self.cnv = cnv
        self.id = self.draw_inplace()
        

    def draw_inplace(self):
        """
        Draw the shape to canvas according to x, y, x_size, y_size attributes.
        Return the shape id.
        """
        return self.create_shape(self.x - self.x_size,
                                 self.y - self.y_size,
                                 self.x + self.x_size,
                                 self.y + self.y_size,
                                 fill = self.color())

    def create_shape(self, *args, **kwargs): return self.cnv.create_oval(*args, **kwargs)
    def color(self): return "white"


class GUIPellet(pm.Pellet, GUIGameObject): pass

class GUIWall(GUIGameObject):

     def create_shape(self, *args, **kwargs): return self.cnv.create_rectangle(*args, **kwargs)
        

class GUIMovingObject(pm.MovingObject, GUIGameObject):
    """
    Adds drawing a move to the move method.
    """

    def move(self, dx=0, dy=0):
        super().move(dx, dy)
        self.draw_move(dx, dy)

    
    def draw_move(self, dx, dy):
        """
        Move the shape on canvas by dx, dy.
        """ 
        self.cnv.move(self.id, dx, dy)


class GUIPacMan(GUIMovingObject, pm.PacMan):

    def color(self): return "yellow"
    
    def transform(self):
        super().transform()
        self.cnv.itemconfig(self.id, fill="red")

    def transform_warning(self):
        super().transform_warning()
        self.cnv.itemconfig(self.id, fill="orange")

    def normalize(self):
        super().normalize()
        self.cnv.itemconfig(self.id, fill=self.color())    

        
class GUIGhost(GUIMovingObject, pm.Ghost):
    def color(self): return "cyan"


class GUIWorldBuilder(pm.WorldBuilder):

    def __init__(self, cnv, **kwargs):
        super().__init__(**kwargs)
        self.cnv = cnv

    def add_obj(self, **kwargs):
        super().add_obj(cnv=self.cnv, **kwargs)

    def init_obj_maps(self):
        self.static_map = {"wall": GUIWall,
                           "pellet": GUIPellet,
                           "superpellet": GUIPellet}
        self.moving_map = {"ghost": GUIGhost,
                           "pacman": GUIPacMan}     
 
  
class GameApp(Canvas):

    def __init__(self, root, *args, **kwargs):
        super().__init__(root, width=700, height=500, bg="blue", *args, **kwargs)
        self.root = root
        self.SPEED = 2
        #self.create_world()
        self.pacman, self.world = pm.standard_world(GUIWorldBuilder(cnv=self))
        self.world.bind("Destroy", lambda obj: self.delete(obj.id))
        self.world.bind("Lose life", self.lose_life)
        self.world.bind("Change score", self.change_score)
        self.world.bind("Game won", self.game_won)
        root.bind("<KeyPress>", self.keypress) # Seome klahvid wasd vastavate callback funktsioonidega, kasutades Tk meetodi bind
        self.font = ("Helvetica", 12, "bold")
        self.score_id = self.create_text(500, 50, fill="yellow", font=("Helvetica", 12, "bold"))
        self.change_score(0)
        self.ticking = True

    def keypress(self, event):
        c = event.char
        if c == "w":
            self.pacman.set_heading(0, -self.SPEED)
        if c == "s":
            self.pacman.set_heading(0, self.SPEED)
        if c == "a":
            self.pacman.set_heading(-self.SPEED, 0)
        if c == "d":
            self.pacman.set_heading(self.SPEED, 0)
      
             
    def timer_tick(self, ms=50): # Reageerime kella tiksumisele ja registreerib järgmise tiksumise
        self.world.tick()
        if self.ticking:
            self.after(ms, self.timer_tick)

    def lose_life(self, **kwargs):
        self.delete(ALL)
        self.create_text(200, 200, fill="yellow", font=self.font, text="Game Over!")
        self.ticking = False

    def change_score(self, score, **kwargs):
        self.itemconfig(self.score_id, text="Score: %i/%i"%(score, self.world.starting_pellets))

    def game_won(self, **kwargs):
        self.delete(ALL)
        self.create_text(200, 200, fill="yellow", font=self.font, text="You win!")
        self.ticking = False
        
        
        
if __name__ == "__main__":
    root = Tk()
    app = GameApp(root)
    app.grid()
    app.timer_tick()
    root.mainloop()
    




